from __future__ import annotations

from prettyqt import widgets
from prettyqt.qt import QtWidgets


class GraphicsPathItem(
    widgets.AbstractGraphicsShapeItemMixin, QtWidgets.QGraphicsPathItem
):
    pass
