from __future__ import annotations

from prettyqt import widgets
from prettyqt.qt import QtWidgets


class StyleOptionGraphicsItem(
    widgets.StyleOptionMixin, QtWidgets.QStyleOptionGraphicsItem
):
    pass
